﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=5.9
@EndOfDesignText@
'wrapped by Anele 'Mashy' Mbanga for ABMaterial
'source: https://www.kalyani.com/blog/2010/10/08/subway-map-visualization-jquery-plugin/
'anele@mbangas.com
'NB: DONT EVER USE NUMERIC MAP KEYS
Sub Class_Globals
	Public ABMComp As ABMCustomComponent
	Private Columns As Int   ' number of columns
	Private Rows As Int  ' number of rows
	Private CellSize As Int  ' the size of each cell
	Private GridNumbers As Boolean  ' show grid numbers
	Private LineWidth As Int  ' the size of each line
	Private ShowGrid As Boolean ' show the grid
	Private page As ABMPage
	Private compID As String
	Private div As HTMLElement
	Private legend As HTMLElement
	Private lines As Map
	Private ReverseMarkers As Boolean
	Private routes As Map
	Type MarkerType(station As String, atstation As String, interchange As String, atinterchange As String, none As String)
	Public EnumType As MarkerType
	Type MarkerDirectionType(N As String, S As String, E As String, W As String)
	Type MarkerPosType(N As String,E As String, S As String, W As String,NE As String,NW As String,SE As String,SW As String)
	Public EnumDirection As MarkerDirectionType
	Public EnumLabelPos As MarkerPosType
	Private mWidth As String
	Private mHeight As String
	Type MarkerInfoType(v1 As String,v2 As String,v3 As String, v4 As String, v5 As String, h1 As String, h2 As String, h3 As String, h4 As String, h5 As String)
	Public EnumInfo As MarkerInfoType
	Private shiftX As String
	Private shiftY As String
	Public Debug As Boolean
	Public HideLabels As Boolean
	Public ToolTip As String
	Private ABM As ABMaterial
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize(InternalPage As ABMPage, ID As String, nRows As Int, nColumns As Int, nCellSize As Int, bGridNumbers As Boolean, bShowGrid As Boolean, nLineWidth As Int, bReverseMarkers As Boolean, Width As String, Height As String)
	ABMComp.Initialize("ABMComp", Me, InternalPage, ID,"")
	page = InternalPage
	compID = ID
	Columns = nColumns
	Rows = nRows
	CellSize = nCellSize
	GridNumbers = bGridNumbers
	ShowGrid = bShowGrid
	div.Initialize(compID,"div")
	LineWidth = nLineWidth
	ReverseMarkers = bReverseMarkers
	routes.Initialize
	lines.Initialize
	EnumType.Initialize 
	EnumType.interchange = "interchange"
	EnumType.station = "station"
	EnumType.atinterchange = "@interchange"
	EnumType.atstation = "@station"
	EnumType.none = ""
	EnumDirection.Initialize
	EnumDirection.E = "E"
	EnumDirection.N = "N"
	EnumDirection.S = "S"
	EnumDirection.W = "W"
	EnumLabelPos.Initialize
	EnumLabelPos.E = "E"
	EnumLabelPos.N = "N"
	EnumLabelPos.S = "S"
	EnumLabelPos.W = "W"
	EnumLabelPos.NE = "NE"
	EnumLabelPos.NW = "NW"
	EnumLabelPos.SE = "SE"
	EnumLabelPos.SW = "Sw"
	EnumInfo.Initialize 
	EnumInfo.h1 = "h1"
	EnumInfo.h2 = "h2"
	EnumInfo.h3 = "h3"
	EnumInfo.h4 = "h4"
	EnumInfo.h5 = "h5"
	EnumInfo.v1 = "v1"
	EnumInfo.v2 = "v2"
	EnumInfo.v3 = "v3"
	EnumInfo.v4 = "v4"
	EnumInfo.v5 = "v5"
	shiftX = ""
	shiftY = ""
	mWidth = Width
	mHeight = Height
	Debug = False
	HideLabels = True
End Sub

'add a new line to the subway, this is the control line
Sub AddLine(lineID As Int, lineColor As String, lineIntensity As String, lineLabel As String, lineShiftX As String, lineShiftY As String, linetextClass As String) As MashSubway
	'use abmaterial methodology for colors
	Dim lineColor As String = MashPlugIns.ABMGetColor(lineColor,lineIntensity)
	Dim lineKey As String = "l" & lineID
	'define the ul element
	Dim ul As HTMLElement
	ul.Initialize(lineKey,"ul")
	ul.AddProperty("data-color",lineColor)
	ul.AddProperty("data-label",lineLabel)
	If linetextClass = "" Then linetextClass = "stext"
	ul.AddProperty("data-textClass", linetextClass)
	Dim lineshiftCoords As String = $"${lineShiftX},${lineShiftY}"$
	If lineshiftCoords <> "," Then
		'data-shiftCoords = -1,0 means move the line to the left by 1 line width.)
		ul.AddProperty("data-shiftCoords",lineshiftCoords)
	End If
	lines.Put(lineKey,ul)
	'create blank routes for the line
	Dim rList As List
	rList.Initialize
	routes.Put(lineKey,rList)
	Return Me
End Sub

'create a subway item
Sub CreateItem(subWay As MashSubway, linePos As Int, X As String, Y As String) As MashSubwayItem
	Dim item As MashSubwayItem
	item.Initialize(subWay,linePos,X,Y)
	Return item 
End Sub

'add points only
Sub AddXY(lineID As Int, X As String, Y As String) As MashSubway
	AddRoute(lineID,x,y,"","","","","","")
	Return Me
End Sub

'add points + direction
Sub AddDirection(lineID As Int, X As String, Y As String, markerDirection As String) As MashSubway
	AddRoute(lineID,x,y,"","","",markerDirection,"","")
	Return Me
End Sub

'add points + direction
Sub AddEast(lineID As Int, X As String, Y As String) As MashSubway
	AddRoute(lineID,x,y,"","","",EnumDirection.E,"","")
	Return Me
End Sub

Sub AddNorth(lineID As Int, X As String, Y As String) As MashSubway
	AddRoute(lineID,x,y,"","","",EnumDirection.N,"","")
	Return Me
End Sub

Sub AddSouth(lineID As Int, X As String, Y As String) As MashSubway
	AddRoute(lineID,x,y,"","","",EnumDirection.S,"","")
	Return Me
End Sub

Sub AddWest(lineID As Int, X As String, Y As String) As MashSubway
	AddRoute(lineID,x,y,"","","",EnumDirection.W,"","")
	Return Me
End Sub


'add points + direction
Sub AddStationWithLabel(lineID As Int, X As String, Y As String, markerLabel As String) As MashSubway
	AddRoute(lineID,x,y,EnumType.station,markerLabel,"","","","")
	Return Me
End Sub


Sub AddStation(lineID As Int, X As String, Y As String) As MashSubway
	AddRoute(lineID,x,y,EnumType.station,"","","","","")
	Return Me
End Sub

Sub AddInterchange(lineID As Int, X As String, Y As String) As MashSubway
	AddRoute(lineID,x,y,EnumType.interchange,"","","","","")
	Return Me
End Sub

Sub AddInterchangeDirection(lineID As Int, X As String, Y As String, markerDirection As String) As MashSubway
	AddRoute(lineID,x,y,EnumType.interchange,"","",markerDirection,"","")
	Return Me
End Sub

Sub AddStationDirection(lineID As Int, X As String, Y As String, markerDirection As String) As MashSubway
	AddRoute(lineID,x,y,EnumType.station,"","",markerDirection,"","")
	Return Me
End Sub

'add a sub way item
Sub AddItem(lineID As Int, item As MashSubwayItem) As MashSubway
	'lets add this route to the corresponding line
	Dim lineKey As String = "l" & lineID
	'get the list
	Dim rrList As List = routes.Get(lineKey)
	'generate a markerID
	Dim markerID As String = rrList.size
	Dim rKey As String = $"${lineKey}.${markerID}"$
	item.li.addproperty("id", rKey)
	item.li.id = rKey
	Dim smarker As String = item.li.HTML.Replace(CRLF,"")
	rrList.Add(smarker)
	routes.Put(lineKey,rrList)
	Return Me	
End Sub

'add a route to the subway: this is a longer method, use the CreateItem short and quicker method
Sub AddRoute(lineID As Int, X As String, Y As String, markerType As String, markerLabel As String, markerLink As String, markerDirection As String, markerLabelPos As String, MarkerInfo As String) As MashSubway
	'lets add this route to the corresponding line
	Dim lineKey As String = "l" & lineID
	'get the list
	Dim rrList As List = routes.Get(lineKey)
	'generate a markerID
	Dim markerID As String = rrList.size
	Dim rKey As String = $"${lineKey}.${markerID}"$
	Dim li As HTMLElement
	li.Initialize(rKey,"li")
	li.AddProperty("data-coords",$"${x},${y}"$)
	If markerType <> "" Then li.AddProperty("data-marker",markerType)
	If markerDirection <> "" Then li.AddProperty("data-dir", markerDirection)   ' direction the line will first go
	If markerLabelPos <> "" Then li.AddProperty("data-labelPos",markerLabelPos)
	If MarkerInfo <> "" Then li.AddProperty("data-markerInfo", MarkerInfo)
	If markerLink <> "" Then
		Dim a As HTMLElement
		a.Initialize($"a${x}${y}"$,"a")
		'a.AddClass("stext")
		'a.AddStyleAttribute("width","100px")
		'a.AddStyleAttribute("text-align","center")
		'a.AddStyleAttribute("margin","12px 0 0 -50px")
		'a.AddProperty("title","")
		'a.AddStyleAttribute("position","absolute")
		'a.AddStyleAttribute("z-index","3000")
		a.SetHREF(markerLink)
		If HideLabels = False Then
			If markerLabel <> "" Then 
				a.AddContent(markerLabel)
				'a.AddAttribute("title",markerLabel,True)
			End If
			
		End If
		li.AddContent(a.HTML.Replace(CRLF,"")) 	
	Else
		If HideLabels = False Then
			If markerLabel <> "" Then 
				li.AddContent(markerLabel)
				'li.AddAttribute("title",markerLabel,True)
			End If
		End If
	End If
	Dim smarker As String = li.HTML.Replace(CRLF,"")
	rrList.Add(smarker)
	routes.Put(lineKey,rrList)
	Return Me
End Sub

'prepare the div to render to
private Sub Prepare
	div.AddClass("subway-map").AddProperty("data-columns",Columns).AddProperty("data-rows",Rows).AddProperty("data-cellSize",CellSize)
	div.AddProperty("data-legendId","legend").AddProperty("data-textClass","stext").AddProperty("data-gridNumbers",GridNumbers).AddProperty("data-grid",ShowGrid)
	div.AddProperty("data-lineWidth", LineWidth).AddProperty("data-reverseMarkers",ReverseMarkers)
End Sub
	
'build the subway	
private Sub BuildSubWay As String
	'we will replace everything on build, lets create the build
	Prepare	
	' add each line to the subway
	For Each strKey As String In lines.Keys
		'get the saved htmlelement
		Dim ul As HTMLElement = lines.Get(strKey)
		'now add the routes
		Dim rrList As List = routes.Get(strKey)
		If rrList.IsInitialized Then
			Dim sroutes As String = MashPlugIns.MvFromList(rrList,CRLF)
			ul.AddContent(sroutes)
		End If
		Dim ssubline As String = ul.HTML
		ssubline = ssubline.Replace(CRLF,"")
		div.AddContent(ssubline).Append(CRLF)
	Next
	'lets create the legend
	legend.Initialize("legend","div")
	Dim sout As String = div.HTML.Replace(CRLF,"")
	sout = sout & legend.HTML.Replace(CRLF,"")
	Return sout
End Sub

'convert a list to an mv

Sub ABMComp_Build(InternalPage As ABMPage,internalID As String) As String
	'lets try and inject the page with css
	'let's use the htmlelement to draw up the build string
	page = InternalPage
	compID = internalID
	
	page.InjectCSS(".stext { text-decoration: none; color: black;}")
	page.InjectCSS($".subway-map { margin: 0; width: ${mWidth}; height:${mHeight}; background-color: white;}"$)
	page.InjectCSS("#legend { float: left; width: 250px; height:400px; }")
	page.InjectCSS("#legend div { height: 25px; }")
	page.InjectCSS("#legend span { margin: 5px 5px 5px 0; }")
	page.InjectCSS(".subway-map span { margin: 5px 5px 5px 0; }")
	page.InjectCSS(".subway-map { font-family: font-family: Verdana; font-size: 8pt;}")
	
	Prepare
	Dim sb As StringBuilder
	sb.Initialize
	sb.Append(div.HTML).Append($"<script>var _${internalID};</script>"$)
	Dim sout As String = sb.ToString
	sout = sout.Replace(CRLF,"")
	Return sout
End Sub

' Is useful to run some initalisation script.
Sub ABMComp_FirstRun(InternalPage As ABMPage, internalID As String)
	Dim script As String = BuildSubWay
	'replace everything in div
	ABM.ReplaceMyInnerHTML(page,internalID,script)
	Dim script As String = $"$(".subway-map").subwayMap({ debug: ${Debug} });"$
	InternalPage.ws.Eval(script, Array As Object(compID))
End Sub

' runs when a refresh is called
Sub ABMComp_Refresh(InternalPage As ABMPage, internalID As String)
'	Dim script As String = BuildSubWay
'	'replace everything in div
'	ABM.ReplaceMyInnerHTML(page,internalID,script)
'	Dim script As String = $"$(".subway-map").subwayMap({ debug: false });"$
'	InternalPage.ws.Eval(script, Array As Object(compID))
End Sub

' do the stuff needed when the object is removed
Sub ABMComp_CleanUp(InternalPage As ABMPage, internalID As String)
End Sub
